<?php

namespace Drupal\ses_email_bounce\Controller;

/**
 * @file
 * Contains \Drupal\ses_email_bounce\Controller\DisplayTableController.
 */

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Serialization\Json;

/**
 * Controller ses email bounce class.
 */
class DisplayTableController extends ControllerBase {

  /**
   * The database connection service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Constructs database connection.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   A connection .
   */
  public function __construct(
    Connection $connection
  ) {
    $this->connection = $connection;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database')
    );
  }

  /***
   * Display log Table.
   */
  public function getLogsTable() {
    $header_table = [
      'id' => $this->t('#ID'),
      'ses_timestamp' => $this->t('Date'),
      'bounced_recipients' => $this->t('Bounce Email Id'),
      'bounce_sub_type' => $this->t('Bounce Sub Type'),
      'log_deatl' => $this->t('Log Detail'),
    ];
    $query = $this->connection->select('ses_email_bounce', 'm');
    $query->fields(
      'm',
      [
        'id',
        'bounce_sub_type',
        'bounced_recipients',
        'ses_timestamp',
      ]
    );
    $results = $query->orderBy('id', 'DESC')
      ->condition('type', 'Notification')
      ->extend('Drupal\Core\Database\Query\PagerSelectExtender')
      ->limit(6)
      ->execute()->fetchAll();
    $rows = [];
    foreach ($results as $data) {
      $id = new FormattableMarkup('<a href="/admin/ses_email_bounce/logs/@id"> @id </a>', ['@id' => $data->id]);
      $detail_link = new FormattableMarkup('<a href="/admin/ses_email_bounce/logs/@id">Log Detail</a>', ['@id' => $data->id]);
      $rows[] = [
        'id' => $id,
        'ses_timestamp' => $data->ses_timestamp,
        'bounced_recipients' => $data->bounced_recipients,
        'bounce_sub_type' => $data->bounce_sub_type,
        'detail_link' => $detail_link,
      ];
    }
    // Bounce Listing.
    $output['table_info'] = [
      '#markup' => '<h3>SES Email Bounce List</h3><br>',
    ];
    $output['ses_log_table'] = [
      '#type' => 'table',
      '#header' => $header_table,
      '#rows' => $rows,
      '#empty' => $this->t('No Log found'),
    ];
    $output[] = ['#type' => 'pager'];
    return $output;
  }

  /**
   * Log Detail.
   */
  public function logDetail($id) {
    $result = $this->connection->query("SELECT * FROM {ses_email_bounce} WHERE id = :id", [':id' => $id])->fetchObject();
    $nodes = Json::decode($result->email_remove_from_node_ids);
    $node_rows = "";
    if (count($nodes) > 0) {
      foreach ($nodes as $node_id => $val) {
        foreach ($val as $field_name => $v) {
          foreach ($v as $v1) {
            foreach ($v1 as $v2) {
              if ($v2['value'] == $result->bounced_recipients) {
                $node_rows .= "Node:" . $node_id . "=>" . "Field:" . $field_name . "=>" . $v2['value'];
              }
            }
          }
        }
      }
    }

    $output['table_info'] = [
      '#markup' => '<h3>AWS Ses Email Bounce Detail</h3><br>',
    ];
    $rows = [
      [
        ['data' => $this->t('Bounced Email Id'), 'header' => TRUE],
        $result->bounced_recipients,
      ],
      [
        ['data' => $this->t('SES Bounce Timestamp'), 'header' => TRUE],
        $result->ses_timestamp,
      ],
      [
        ['data' => $this->t('Bounce Type'), 'header' => TRUE],
        $result->bounce_type,
      ],
      [
        ['data' => $this->t('Bounce Sub Type'), 'header' => TRUE],
        $result->bounce_sub_type,
      ],
      [
        ['data' => $this->t('Email Removed From Node Fields'), 'header' => TRUE],
        $node_rows,
      ],
      [
        [
          'data' => $this->t('Email Removed From webform Handlers'),
          'header' => TRUE,
        ],
        $result->email_remove_from_webform_handlers,
      ],
      [
        [
          'data' => $this->t('AWS SES Bounce Message Body (json)'),
          'header' => TRUE,
        ],
        $result->ses_message_body,
      ],
    ];
    $output['ses_log_detail_table'] = [
      '#type' => 'table',
      '#rows' => $rows,
    ];
    return $output;
  }

  /**
   * Get SubscriptionConfirmation.
   */
  public function getSubscriptions() {
    $query = $this->connection->select('ses_email_bounce', 'm');
    $query->fields(
      'm',
      [
        'id',
        'type',
        'subscribe_url',
        'ses_timestamp',
      ]
    );
    $results = $query->orderBy('id', 'DESC')
      ->condition('type', 'SubscriptionConfirmation')
      ->execute()->fetchAll();
    $output['table_info'] = [
      '#markup' => '<h3>API End Point Subscription Confirmation Action</h3><br>',
    ];
    $rows = [];
    foreach ($results as $data) {
      $rows[] = [
        'ses_timestamp' => $data->ses_timestamp,
        'subscribe_url' => $data->subscribe_url,
      ];
    }

    $header_table = [
      'ses_timestamp' => $this->t('Date'),
      'subscribe_url' => $this->t('Subscription Confirmation Action'),
    ];
    $output['ses_log_table'] = [
      '#type' => 'table',
      '#header' => $header_table,
      '#rows' => $rows,
      '#empty' => $this->t('No Log found'),
    ];
    return $output;
  }

}
